import yaml from 'js-yaml'
import markdown, { utilLoaders as markdownUtils, profiles as markdownProfiles } from '../transformers/markdown'
import rehype, { profiles as rehypeProfiles } from '../transformers/rehype'
import retext, { profiles as retextProfiles } from '../transformers/retext'
import { testRule } from 'skypager-util'
import { kebabCase, camelCase, snakeCase } from 'skypager-util/lib/string'
import visit from 'unist-util-visit'
import { lazy } from 'skypager-util'
import omit from 'lodash/omit'
import get from 'lodash/get'
import lodash from 'lodash'

export const rules = [
  /\.md$/
]

export const transformPresets = () => {
  return {
    markdown: Object.keys(markdownProfiles),
    rehype: Object.keys(rehypeProfiles),
    retext: Object.keys(retextProfiles)  
  }
}

export const getTransforms = () => {
  return {
    markdown: require('../transformers/markdown').default,
    rehype: require('../transformers/rehype').default,
    retext: require('../transformers/retext').default,
  }
}

export const getUtils = () => {
  const loaders = markdownUtils()

  return Object.keys(loaders).reduce((memo, key) => (
    Object.assign(memo, {
      get [key]() {
        return loaders[key]()
      }
    })
  ), {})
}

export const testDocument = (doc) => (
  typeof rules.find(rule => testRule(rule, doc.fileBaseName)) !== 'undefined'
)

export const decorate = (doc) => {
  doc.lazy('exportables', () => doc.createExportable())
 
  Object.assign(doc, {
    createExportable(options = {}) {
      const ast = doc.markdown(options.profile || 'parser', options)
      const html = doc.markdown('html', options)

      const exportable = {
        id: doc.id,
        get contents() {
          return doc.readFile().getFileContents()
        },
        get frontmatter() {
          return doc.readFrontmatter()             
        },
        get ast() {
          return ast.parse(exportable.contents)
        },
        get file() {
          return omit(doc.file, 'contents')
        },
        get html() {
          return html.stringify(exportable.ast)
        },
        get headingsMap() {
          return doc.createHeadingsMap()
        },
        get codeBlocks() {
          return doc.chain
            .invoke('selectCodeBlocks', 'javascript', ', code[lang="jsx"], code[lang="react"]')
            .map(v => ({
              heading: v.getHeadingPath().split('/').pop(),
              index: v.index,
              value: v.value,            
              docId: doc.id,
              docPath: v.getHeadingPath(),
              lang: v.lang,
            }))
            .groupBy('docPath')
        }
      }

      return exportable
    },

    readFrontmatter(options = {}) {
      const { contents = doc.getFileContents() } = options
      const parser = doc.markdown(options.parser || 'parser')
      const { children = [] } = parser.parse(contents)

      try {
        if (children[0] && children[0].type === 'yaml') {
          return yaml.load(children[0].value || '')
        } else {
          return {}
        }      
      } catch(e) {
        return {}
      } 
    },

    retextAnalysis(options = {}) {
      const profile = options.profile || 'rehype'
      const parser = doc.markdown(options.parser || 'parser', options)
      const contents = options.contents || doc.getFileContents()
      const ast = options.ast || parser.parse(contents)

      return doc.retext('retext', options).process(ast)
    }, 

    rehypeDocument(options = {}) {
      const parser = doc.markdown(options.parser || 'parser', options)
      const contents = options.contents || doc.getFileContents()
      const ast = options.ast || parser.parse(contents)

      return doc.rehype(profile, options).run(ast)     
    },

    remarkAST(options = {}) {
      const profile = options.profile || 'parser'
      const contents = options.contents || this.getFileContents()

      return doc.markdown(profile, options).parse(contents)    
    },

    remarkHTML(options = {}) {
      const profile = options.profile || 'github'
      const parser = options.parser || doc.markdown(profile, options)
      const contents = options.contents || this.getFileContents()
      const ast = options.ast || parser.parse(contents)
      const html = 'html'

      return doc.markdown(html, options).stringify(ast) 
    },

    retext(profile = 'retext', options) {
      return retext({
        profile,
        ...options,
      })
    },

    rehype(profile = 'rehype', options) {
      return rehype({
        profile,
        ...options,
      })
    },

    markdown(profile = 'parser', options) {
      return markdown({
        profile,
        ...options
      })
    },
    
    createHeadingsMap() {
      return this.chain.invoke('selectHeadingNodes').keyBy(heading => {
        const node = heading.children.find(n => n.type === 'text')
        const value = (node && node.value) || ''

        return camelCase(snakeCase(value)) 
      }).value()
    },

    stringifyNode(node) {
      return require('mdast-util-to-string')(node)
    },

    getSourceForNode(node) {
      return require('unist-util-source')(node)
    },

    decorateNode(node) {
      return Object.assign(node, {
        textContent() {
          return doc.stringifyNode(node) || ''
        }, 

        getHeadingPath() {
          if (node.type !== 'heading') {
            const prev = node.previousHeading() 
            return prev && prev.getHeadingPath()
          }          

          const textContent = kebabCase(node.textContent().toLowerCase())
          const prevSuperior = node.getPreviousSuperior()
          const previousPath = typeof prevSuperior === 'undefined' ? `` : prevSuperior.getHeadingPath()

          return [previousPath, textContent].filter(i => i && i.length > 0).join('/')
        },

        getFinalHeading() {
          return doc.findAllNodesAfter(node, (n) => n.type === 'heading' && n.depth <= node.depth )[0]
        },

        getPreviousSuperior() {
          return node.getNodesBefore(n => n.type === 'heading' && (n.depth < node.depth))[0]
        },

        previousHeading() {
          return node.getNodesBefore(n => n.type === 'heading').shift()
        },

        nextHeading() {
          return node.type === 'heading' 
            ? node.getNextNode((n) => n.type === 'heading' && n.depth >= node.depth)
            : node.getNextNode(n => n.type === 'heading') 
        },

        getNextNode(...args) {
          return doc.findNodeAfter(node, ...args)
        },

        getNodesAfter(...args) {
          return doc.findAllNodesAfter(node, ...args)
        },

        getNodesBefore(...args) {
          return doc.findAllNodesBefore(node, ...args)
        },
      })
    },

    wrapSections() {
      const asts = this.sectionHeadingNodes()
        .map(node => this.createAstFromSection(node, {
          offset: false,
          type: 'div',
          data: {
            hName: 'section',
            hProperties: {
              'data-section-path': node.getHeadingPath(),
              'data-node-index': node.index,
            },
          }
        }))
      

      return asts
    },

    organizeAsModuleExport() {
      const headings = this.selectNodes(`code ~ heading[depth=2], code ~ heading[depth=3], code ~ heading[depth=4]`)      

      return this.chain.plant(headings)
        .sortBy(h => h.index)
        .keyBy(h => h.getHeadingPath())
        .map(h => this.createAstFromSection(h))
        .mapValues(ast => require('unist-util-select')(ast)('code[lang^=javascript]'))
        .mapValues(code => code.map(c => c.value))
        .value()
    },

    createDocumentFromSection(node) {
      const ast = this.createAstFromSection(node)

      const contents = this.project.transformer('markdown', {
        parser: 'stringifier'
      }).render(ast)

      return {
        id: `${this.id}/~/${node.getHeadingPath()}`,
        contents,
        ast,    
      }
    }, 

    createAstFromSection(node, options = {}) {
      const startDepth = node.depth
      const offset = options.offset !== false 
        ? 1 - startDepth 
        : 0

      const children = doc.extractSectionNodes(node).map(child => 
        child.type === 'heading' ? ({...child, depth: child.depth + offset}) : child
      )

      const newAST = {
        type: options.type || 'root',
        children,
        data: options.data || {},
      }

      return newAST
    },

    extractSectionNodes(node) {
      const startAt = node.index
      const finalHeading = node.getFinalHeading()
      const endAt = finalHeading 
        ? finalHeading.index 
        : doc.finalHeading() || Infinity 

      return [node, ...this.selectNodes('*', n => n.index > startAt && n.index < endAt)]
    },

    inspectOutline() {
      const headings = this.selectHeadingNodes()

      return headings.map(headingNode => {
        const content = `- ${headingNode.textContent()}` 
        return lodash.padStart(content, content.length + (headingNode.depth - 1) * 2)         
      }).join("\n")
    }, 

    headingIndexes(depth) {
      return this.chain
        .invoke('selectHeadingNodes', depth && `[depth=${depth}]`)
        .sortBy(node => node.position ? node.position.start.line : node.index)
        .map(node => node.index)
        .value()
    },

    selectHeadingsByDepth(depth = 1) {
      return this.selectHeadingNodes(`[depth=${depth}]`)
    },

    selectOneNode(selector) {
      return require('unist-util-select')(this.ast)(selector)[0]
    },

    selectNodes(selector = '*', testFn) {
      const results = require('unist-util-select')(this.ast)(selector)
      
      return typeof testFn === 'function' 
        ? results.filter(r => testFn.call(doc, r))
        : results
    }, 

    selectHeadingNodes(enhance = '') {
      return doc.selectNodes(`heading${enhance}`.trim())
    },

    selectCodeBlocks(lang = 'javascript', enhance='') {
      return doc.selectNodes(`code[lang=${lang}]${enhance}`.trim())
    },

    findNodeAfter(node, ...args) {
      return require('unist-util-find-after')(this.ast, node, ...args)
    },

    findAllNodesAfter(node, ...args) {
      return require('unist-util-find-all-after')(this.ast, node, ...args)
    },

    findAllNodesBefore(node, ...args) {
      return require('unist-util-find-all-before')(this.ast, node, ...args)
    },

    nodesIndexedByPosition() {
      return doc.chain
        .get('ast.children')
        .keyBy(node => parseInt(node.position.start.line))
        .value()
    },

    introNodes() {
      const f = this.selectNodes('heading')
      const m = f.find(n => n.depth > 1)

      return m ? m.getNodesBefore().reverse() : this.ast.children
    },

    finalHeading() {
      return this.selectNodes('heading:last-of-type')
    },
    
    titleNode() {
      return this.selectOneNode('heading[depth=1]') || this.selectOneNode('heading:first-of-type')
    },

    firstSectionHeadingNode() {
      return this.selectOneNode('heading[depth=2]')
    },

    sectionHeadingNodes() {
      return this.selectNodes('heading[depth=2]')
    },

    articleHeadingNodes() {
      return this.selectNodes('heading[depth=3]')
    },

    subarticleHeadingNodes() {
      return this.selectNodes('heading').filter(n => n.depth >= 4)
    },

    createCodeRunners (context = {}) {
      return this.chain.get('headingsMap')
        .mapValues((heading) => heading.getNextNode('code'))
        .pickBy((node) => node && node.lang === 'javascript')
        .mapValues(node => this.project.createAsyncScriptRunner(node.value, {}, {...context, doc: this, project: this.project}))
        .value()
    },

    getNodesBetweenPositions(alpha, bravo) {
      const start = isNode(alpha) ? alpha.position.start.line : parseInt(alpha)
      const end = isNode(bravo) ? bravo.position.start.line : parseInt(bravo)

      return doc.chain.get('ast.children').filter(node => {
        const { position: {
          start: {
            line
          }
        }} = node

        return line > start && line < end
      })
    },

    finalNode() {
      return this.chain.get('ast.children', []).last().value() 
    },

    getAST() {
      return doc.project.cache.fetch(`asts:${doc.cacheKey}`, () => {
        const _ast = doc.remarkAST()

        return {
          ..._ast,
          children: _ast.children.map((n,i) => {
            const d = doc.decorateNode(n)
            d.index = i
            return d
          }),
        }
      })
    },
  })

  doc.lazy('ast', () => doc.getAST())
  doc.lazy('headingsMap', () => doc.createHeadingsMap())
  
  return doc
}

export default decorate

function isNode(obj) {
  return typeof obj === 'object' && typeof obj.type === 'string' && obj.position
}
