import url from 'url'
import { basename, resolve, relative } from 'path'
import { mixinPropertyUtils, strings as stringUtils } from 'skypager-util'
import isAbsolutePath from 'path-is-absolute'
import Collection from '../collection'
import mapValues from 'lodash/mapValues'
import isFunction from 'lodash/isFunction'
import vfile from 'vfile'
import get from 'lodash/get'
import result from 'lodash/result'
import set from 'lodash/set'
import omit from 'lodash/omit'
import pick from 'lodash/pick'
import uniq from 'lodash/uniq'

export const DefaultPatterns = [
  '**/*.{js,md,yml,json,svg,html,css,scss,less,json}',
  '!node_modules/**'
]

export const patterns = DefaultPatterns

export class Document {
  static patterns = DefaultPatterns;
 
  static mountCollection(cwd = process.cwd(), options = {}) {
    return Collection.mount(cwd, options)
  }

  constructor (vFileOrPath, options = {}) {
    mixinPropertyUtils(this)

    const originalURI = normalizeUri(
      vFileOrPath.path ? vFileOrPath.path : vFileOrPath,
      options.cwd || process.cwd()
    )

    if (options.project) {
      this.hide('project', options.project)
      delete(options.project)
    }

    if (options.collection) {
      this.hide('collection', options.collection)
      delete(options.collection)
    }

    this.hide('options', options)
    this.hide('cwd', options.cwd)
    this.hide('originalURI', originalURI)
    this.hide('parsedURI', url.parse(this.originalURI))
    this.hide('url', this.parsedURI.href)
    this.hide('uri', this.url)
    this.hide('path', unescape(this.parsedURI.path))

    if (vFileOrPath.path) {
      this.options.file = vFileOrPath
    }

    this.lazy('file', () => this.toVFile({
      cwd: this.cwd,
      ...(this.options.file || {})
    }))

    this.hide('data', {})
    this.lazy('id', this.buildId.bind(this))

    this.lazy('fileStat', () => this.get('file.stat', {}))
    this.lazy('createdAt', () => this.result('fileStat.ctime', () => Math.floor(new Date() / 1000)))
  }

  get updatedAt() {
    return Math.floor(this.get('data.updatedAt', this.createdAt) / 1000)
  }

  touch() {
    try {
      this.set('data.updatedAt', Math.floor(new Date() / 1000))
    } catch(e) {

    }

    return this
  }

  get decorated() {
    this.applyDoctypes()
    return this
  } 

  getTransformPresets() {
    return this.matchingDocTypes
      .reduce((memo, docType) => ({
        ...memo,
        ...docType.getTransforms ? docType.getTransforms() : {},
      }))  
  }
  
  getTransforms() {
    return this.matchingDocTypes
      .reduce((memo, docType) => ({
        ...memo,
        ...docType.getTransforms ? docType.getTransforms() : {},
      }))
  }  
  
  getUtils() {
    return this.matchingDocTypes
      .reduce((memo, docType) => ({
        ...memo,
        ...docType.getUtils ? docType.getUtils() : {},
      }))
  }

  buildId () {
    if (isFunction(this.options.buildId)) {
      return this.options.buildId(
        relative(this.cwd, this.path)
      )
    } else if (this.collection && isFunction(this.collection.buildId)) {
      return this.collection.buildId(
        relative(this.cwd, this.path)
      )
    } else {
      return relative(this.cwd, this.path)
        .replace(/\.\w+$/g, '')
        .replace(/\.js$/g,'')
    }
  }

  get chain() {
    return this.project.chain.plant(this)
  }

  get exporter() {
    return this.project.chain.plant(this.ready)
  }

  get ready() {
    return this.readFile(true).decorated
  }

  get doctypes() {
    return this.docTypes
  }

  get docTypes() {
    const { project } = this

    return project.chain
      .get('documentTypes.available')
      .thru(list => list.map(docType => project.documentType(docType)))  
      .value()
  }

  get matchingDocTypes() {
    const { project } = this

    return project.chain
      .get('documentTypes.available')
      .thru(list => list.map(docType => project.documentType(docType)))  
      .flatten()
      .filter(docType => docType.testDocument(this))
      .value()  
  }

  get idParts() {
    return this.id.split('/')
  }

  get baseFolder() {
    return this.idParts[0]
  }

  get categoryFolderName() {
    return this.idParts[1]
  }

  get categoryFolder() {
    return this.idParts.slice(0, 2).join('/')
  }

  applyDoctypes() {
    if (this.doctypesApplied) {
      return this      
    } else {
      this.matchingDocTypes.forEach(docType => {
        docType.decorate(this)
      })    

      this.doctypesApplied = true

      return this
    }
  }

  toJson (options = {}) {
    return {
      id: this.id,
      file: omit(this.file, 'contents', '_contents'),
      metadata: this.metadata,
      idParts: this.idParts,
      categoryFolder: this.categoryFolder,
      categoryFolderName: this.categoryFolderName,
      baseFolder: this.baseFolder,
      variations: this.variations,
      ...(options.pick ? pick(this, options.pick) : {}),
      ...(options.pickBy ? pickBy(this, options.pickBy) : {}),
      ...(options.includeContents ? {contents: this.file.contents} : {contents: null}),
    }
  }

  toVFile(fileOptions = {}) {
    return vfile({
      path: this.path,
      ...fileOptions,
    })
  }

  readFile(cache = true) {
    if (cache && this.contents && this.contents.length > 0) {
      this.file.contents = this.contents      
      return this
    }

    this.file.contents = this.contents = this.project.readFileSync(this.file.path).toString()

    return this.touch() 
  }

  readFileAsync() {
    return this.project.readFileAsync(this.file.path)
      .then((contents) => this.contents = contents.toString())
  }

  get metadata() {
    return {
      ...this.fileData,
      ...this.data,
      ...this.get('exportables.frontmatter', {}),
    }
  }

  get idPair() {
    return [this.id, this]
  }

  get isIndex() {
    return !!(this.id.match(/index$/) || this.file.basename === 'index.js')
  }

  get isPackageManifest() {
    return this.fileBaseName === 'package.json'
  }

  get variationsBase() {
    return this.isIndex
      ? this.fileDirnameBasename
      : this.fileStem
  }

  get fileExtname() {
    return this.file.extname
  }

  get isMarkdown() {
    return this.fileExtname && this.fileExtname.match(/.md$/)
  }
  
  get isJavascript() {
    return this.fileExtname && this.fileExtname.match(/.js$/)
  }

  createStemVariations (base = this.variationsBase, options = {}) {
    const { startCase, camelCase, kebabCase, snakeCase } = stringUtils

    return mapValues({
      classified: startCase(camelCase(base)).replace(/\s+/g,''),
      camel: camelCase(base),
      kebab: kebabCase(base),
      snake: snakeCase(base),
      underscored: kebabCase(base).replace(/\-/g,'_')
    }, (v) => {
      const value = `${options.prefix || ''}${v}${options.suffix || ''}`
      const modifier = options.pluralize
        ? stringUtils.pluralize
        : options.singularize ? stringUtils.singularize : ((v) => v)

      return modifier(value)
    })
  }

  get fileData () {
    return isFunction(this.options.getFileData)
      ? this.options.getFileData.call(this, this.file)
      : this.result('file.data', () => Object.assign(this.file, {
        data: {},
      }))
  }

  get fileMessages () {
    return this.get('file.messages', {})
  }

  get variations () {
    return {
      ...this.createStemVariations(this.variationsBase),
      singularized: this.createStemVariations(this.variationsBase, {
        singularize: true,
      }),
      pluralized: this.createStemVariations(this.variationsBase, {
        pluralize: true,
      }),
    }
  }

  get nameVariations() {
    const variations = this.variations

    return uniq(Object.values(omit(variations, 'pluralized', 'singularized')).concat(
      Object.values(variations.pluralized),
      Object.values(variations.singularized)
    ))
  }

  get indexVariations() {
    return this.createStemVariations(this.variationsBase, {
      suffix: `/index${this.fileExtname}`,
    })
  }

  get filenameVariations() {
    return {
      ...this.createStemVariations(this.variationsBase, {
        suffix: this.fileExtname,
      })
    }
  }

  get fileStem() {
    return this.file.stem
  }

  get fileBaseName() {
    return this.file.basename
  }

  get fileDirname () {
    return this.file.dirname
  }

  get fileDirnameBasename () {
    return basename(this.fileDirname)
  }

  get baseRelativePath() {
    return relative(
      this.cwd,
      this.path
    )
  }

  get contentLength() {
    return parseInt(this.result(
      'stat.size', 
      () => this.get('contents', this.get('file.contents', '')).toString().length
    ))
  }

  get stat() {
    return this.get('file.stat', {})
  }

  get cacheKey() {
    return [
      this.baseRelativePath, 
      this.updatedAt.toString(), 
      this.contentLength.toString()
    ].join(':')
  }

  get isRemote() {
    return this.uri.protocol.indexOf('http') !== -1
  }
  
  reloadFileContents() {
    return this.contents = this.file.contents = this.readFileSync(false)
  }

  getFileContents() {
    return this.result('file.contents', '').toString()
  }
}

export default Document

export function normalizeUri (input, cwd = process.cwd()) {
  if (typeof input !== 'string') {
    throw('Must pass the URI as a string')
  }

  const parsed = url.parse(input)

  const isFilePath = parsed.protocol === null || parsed.protocol.indexOf('file') !== -1
  const isRemoteUrl = !isFilePath && parsed.protocol.indexOf('http') !== -1
  const isAbsoluteFilePath = isFilePath && isAbsolutePath(parsed.path)

  if (isRemoteUrl) {
    return url.format(parsed)
  }

  if (isFilePath && isAbsoluteFilePath) {
    return url.format({
      protocol: 'file:',
      slashes: true,
      pathname: parsed.path,
    })
  } else {
    return url.format({
      protocol: 'file:',
      slashes: true,
      pathname: resolve(cwd, parsed.path),
    })
  }

  return input
}
