import toEntity from './entity'

export default (obj) => {
  const doc = toEntity(obj)

  return Object.assign(doc, {
    createHeadingsMap() {
      return doc.chain.invoke('selectHeadingNodes').keyBy(heading => {
        const node = heading.children.find(n => n.type === 'text')
        const value = (node && node.value) || ''

        return camelCase(snakeCase(value)) 
      }).value()
    },

    stringifyNode(node) {
      return require('mdast-util-to-string')(node)
    },

    getSourceForNode(node) {
      return require('unist-util-source')(node)
    },

    decorateNode(node) {
      return Object.assign(node, {
        textContent() {
          return doc.stringifyNode(node) || ''
        }, 

        getHeadingPath() {
          if (node.type !== 'heading') {
            const prev = node.previousHeading() 
            return prev && prev.getHeadingPath()
          }          

          const textContent = kebabCase(node.textContent().toLowerCase())
          const prevSuperior = node.getPreviousSuperior()
          const previousPath = typeof prevSuperior === 'undefined' ? `` : prevSuperior.getHeadingPath()

          return [previousPath, textContent].filter(i => i && i.length > 0).join('/')
        },

        getFinalHeading() {
          return doc.findAllNodesAfter(node, (n) => n.type === 'heading' && n.depth <= node.depth )[0]
        },

        getPreviousSuperior() {
          return node.getNodesBefore(n => n.type === 'heading' && (n.depth < node.depth))[0]
        },

        previousHeading() {
          return node.getNodesBefore(n => n.type === 'heading').shift()
        },

        nextHeading() {
          return node.type === 'heading' 
            ? node.getNextNode((n) => n.type === 'heading' && n.depth >= node.depth)
            : node.getNextNode(n => n.type === 'heading') 
        },

        getNextNode(...args) {
          return doc.findNodeAfter(node, ...args)
        },

        getNodesAfter(...args) {
          return doc.findAllNodesAfter(node, ...args)
        },

        getNodesBefore(...args) {
          return doc.findAllNodesBefore(node, ...args)
        },
      })
    },

    extractSectionNodes(node) {
      const startAt = node.index
      const finalHeading = node.getFinalHeading()
      const endAt = finalHeading 
        ? finalHeading.index 
        : doc.finalHeading() || Infinity 

      return [node, ...doc.selectNodes('*', n => n.index > startAt && n.index < endAt)]
    },

    inspectOutline() {
      const headings = doc.selectHeadingNodes()

      return headings.map(headingNode => {
        const content = `- ${headingNode.textContent()}` 
        return lodash.padStart(content, content.length + (headingNode.depth - 1) * 2)         
      }).join("\n")
    }, 

    headingIndexes(depth) {
      return doc.chain
        .invoke('selectHeadingNodes', depth && `[depth=${depth}]`)
        .sortBy(node => node.position ? node.position.start.line : node.index)
        .map(node => node.index)
        .value()
    },

    selectHeadingsByDepth(depth = 1) {
      return doc.selectHeadingNodes(`[depth=${depth}]`)
    },

    selectOneNode(selector) {
      return require('unist-util-select')(doc.ast)(selector)[0]
    },

    selectNodes(selector = '*', testFn) {
      const results = require('unist-util-select')(doc.ast)(selector)
      
      return typeof testFn === 'function' 
        ? results.filter(r => testFn.call(doc, r))
        : results
    }, 

    selectHeadingNodes(enhance = '') {
      return doc.selectNodes(`heading${enhance}`.trim())
    },

    selectCodeBlocks(lang = 'javascript', enhance='') {
      return doc.selectNodes(`code[lang=${lang}]${enhance}`.trim())
    },

    findNodeAfter(node, ...args) {
      return require('unist-util-find-after')(doc.ast, node, ...args)
    },

    findAllNodesAfter(node, ...args) {
      return require('unist-util-find-all-after')(doc.ast, node, ...args)
    },

    findAllNodesBefore(node, ...args) {
      return require('unist-util-find-all-before')(doc.ast, node, ...args)
    },

    nodesIndexedByPosition() {
      return doc.chain
        .get('ast.children')
        .keyBy(node => parseInt(node.position.start.line))
        .value()
    },

    introNodes() {
      const f = doc.selectNodes('heading')
      const m = f.find(n => n.depth > 1)

      return m ? m.getNodesBefore().reverse() : doc.ast.children
    },

    finalHeading() {
      return doc.selectNodes('heading:last-of-type')
    },
    
    titleNode() {
      return doc.selectOneNode('heading[depth=1]') || doc.selectOneNode('heading:first-of-type')
    },

    firstSectionHeadingNode() {
      return doc.selectOneNode('heading[depth=2]')
    },

    sectionHeadingNodes() {
      return doc.selectNodes('heading[depth=2]')
    },

    articleHeadingNodes() {
      return doc.selectNodes('heading[depth=3]')
    },

    subarticleHeadingNodes() {
      return doc.selectNodes('heading').filter(n => n.depth >= 4)
    },

    getNodesBetweenPositions(alpha, bravo) {
      const start = isNode(alpha) ? alpha.position.start.line : parseInt(alpha)
      const end = isNode(bravo) ? bravo.position.start.line : parseInt(bravo)

      return doc.chain.get('ast.children').filter(node => {
        const { position: {
          start: {
            line
          }
        }} = node

        return line > start && line < end
      })
    },

    finalNode() {
      return doc.chain.get('ast.children', []).last().value() 
    }
  })
}