/**
 * As powerful as Webpack is, that it removes our ability to perform dynamic require statements
 * is a detriment to the elegance of the code we write. Readability for human beings is just
 * as important as readability for the robots.
 *
 * The Resolver provides us with the best of both words by letting us write dynamic require
 * statements without disrupting Webpack's strict static analysis requirements.
 *
 * This is useful for things like mocking and stubbing tests, and also
 * for being able to write require statements that point to the desired
 * component, and not having to rewrite them all over the place if ever you decide
 * to move a file somewhere else in the tree.
 *
 * Using this system, require('actions/billing') and require('containers/Billing/actions') will
 * statically resolve to the same module for Webpack, and you as the developer can do whatever is
 * most the natural depending on how and where you are doing the requiring.
 *
 * As always exercise caution.  Knives are sharp.  Don't get cut.
 */

const RequireContexts = {
 /**
  * Gives us the Webpack Module Ids of all of the react components
  */
  get components() {
    return require.context('.', true, /\/components\/.*\/index.(js|jsx)$/)
  },

 /**
  * Gives us the Webpack Module Ids of all of the react components
  */
 get containers() {
   return require.context('.', true, /\/containers\/.*\/index.(js|jsx)$/)
 },

 /**
  * Gives us the Webpack Module Ids of all files named reducer.js
  * that live next to a React component. e.g. containers/Billing/reducer
  */
  get reducers() {
    return require.context('.', true, /\/\w+s\/.*\/reducer.js$/)
  },

 /**
  * Gives us the Webpack Module Ids of all files named route.js
  * that live next to a React component. e.g. containers/Login/route.
  */
 get routes() {
   return require.context('.', true, /\/\w+s\/.*\/route.js$/)
 },
}


export class Resolver {
   constructor (dictionary) {
     defineProperty(this, 'dictionary', {
       enumerable: false,
       get: () => dictionary,
     })
   }

   /**
    * Returns a list of all of the known scopes or categories in the dictionary.
    * e.g ['actions', 'reducers', 'routes', 'containers', 'components', 'services']
    */
   get scopes() {
      return keys(this.dictionary)
   }

   resolve (userRequest) {
     const [scope, ...members] = userRequest.split('/')

     if (this.scopes.indexOf(scope) === -1) {
       throw(errors.INVALID_SCOPE_ERROR)
     }
   }
 }

 export default new Resolver(RequireContexts)

 const { defineProperty } = Object
